# -*- coding: utf-8 -*-
"""
DataExtensionImage represents a single image, e.g. a FITS extension.
"""

import numpy

from micado.main.DataExtension import DataExtension, persistent
from micado.hardware.Detector import Detector


class DataExtensionImage(DataExtension):
    """
    DataExtensionImage represents a single image, e.g. a FITS extension.
    """

    # Persistent properties correspond to header keywords.
    NAXIS1: int = persistent('Length of data in axis 1 [pixel]', int, -1)
    NAXIS2: int = persistent('Length of data in axis 2 [pixel]', int, -1)
    EXTNAME: str = persistent('Extension name, from Detector [None]', str, '')
    INHERIT: str = persistent('Inherit keywords from primary [None]', str, 'T')

    image: numpy.ndarray = None
    # The image data

    def __init__(self, data=None, header=None, detector: Detector = None, *args, **kwargs):
        """
        Initialize.
        """
        super().__init__(header=header, *args, **kwargs)
        self.image = data

        if detector is not None:
            if self.NAXIS1 < 0:
                self.NAXIS1 = 2 * int(numpy.round(detector.xhw / detector.pixsize))
            if self.NAXIS2 < 0:
                self.NAXIS2 = 2 * int(numpy.round(detector.yhw / detector.pixsize))
            if self.EXTNAME == '':
                self.EXTNAME = detector.id


class DataExtensionVariance(DataExtensionImage):
    """
    Variance layer of the image.
    """


class DataExtensionQuality(DataExtensionImage):
    """
    Quality layer of the image.
    """
