# -*- coding: utf-8 -*-
"""
The base for all classes modeling hardware.
"""
from collections import OrderedDict
from micado.main.MObject import MObject, persistent


def get_attr_dictionary(attr):
    """
    Get ScopeSIM dictionary for attr.
    """
    if isinstance(attr, Hardware):
        d = attr.scopesim_dictionary
    elif isinstance(attr, list):
        # TODO: this only works for lists of Hardware, not lists of values
        if len(attr):
            lds = [attri.scopesim_dictionary for attri in attr]
            lds0a = lds[0]['kwargs']
            da = OrderedDict([(
                k, [d2['kwargs'][k] for d2 in lds]
            )
                for k in lds0a
            ])
            d = lds[0]
            d['class'] += "List"
            d['kwargs'] = {'array_dict': da}
        else:
            # TODO: test this
            d = None
    else:
        d = attr

    return d


class Hardware(MObject):
    """
    The base for all classes modeling hardware.
    """

    dummy = persistent("dummy", str, "")

    @property
    def scopesim_dictionary(self):
        """
        Create dictionary for ScopeSIM.
        """
        attrnames = [
            attrname for attrname in self.get_persistent()
            if not attrname.startswith('dummy') and attrname not in [
                'object_id', 'name'
            ]]

        attrs = OrderedDict([(
            attrname,
            self.__getattribute__(attrname)
        )
            for attrname in attrnames
        ])

        dict_attr = OrderedDict([(
            attrname,
            get_attr_dictionary(attr),
        ) for attrname, attr in attrs.items()])

        dd = {
            'class': self.__class__.__name__,
            'description': self.__doc__.strip(),
            'kwargs': dict_attr,
        }
        if hasattr(self, 'name'):
            dd['name'] = self.name

        return dd
