"""
This module defines the Imstat object.
The Imstat object is used to store image statistics.
"""

import numpy

from micado.main.MObject import MObject, persistent
from micado.main.BaseFrame import BaseFrame


class Imstat(MObject):
    """
    Imstat contains image statistics.

    Based on astro.main.Imstat but pointing the other way around.
    """

    baseframe: BaseFrame = persistent("The frame [None]", BaseFrame, None)

    min: float = persistent('Minimum value of data [ADU]', float, 0)
    max: float = persistent('Maximum value of data [ADU]', float, 0)
    mean: float = persistent('Mean value of data [ADU]', float, 0)
    std: float = persistent('Standard deviation of data [ADU]', float, 0)
    median: float = persistent('Median of data [ADU]', float, 0)

    def make(self):
        """
        Calculate the statistics.
        """
        images = [ext.image for ext in self.baseframe.extensions]
        self.min = float(numpy.min(images))
        self.max = float(numpy.max(images))
        self.mean = float(numpy.mean(images))
        self.std = float(numpy.std(images))
        self.median = float(numpy.median(images))
