# -*- coding: utf-8 -*-
"""
Observing Blocks
"""

from micado.main.MObject import MObject, persistent
from micado.main.Template import Template
from typing import List


class ObservingBlock(MObject):
    """
    An Observing Block.
    """
    # TODO: Add ObservingBlock to vodsl.

    name: str = persistent("Name for this OB", str, "")

    target: str = persistent("Target for this OB", str, "")

    instrument: str = persistent("Instrument to use", str, "")

    # TODO: Only single templates are allowed at the moment.
    templates: List[Template] = persistent("Templates to observe", Template, [])

    def real_fits_keywords(self, ppp=None):
        """
        Overloaded to not list the TPL keywords explicitly.
        """
        # TODO: Only one template is allowed for now and sholud also be referenced in the raw.
        attrs1 = MObject.real_fits_keywords(self, ppp=ppp)
        attrs2 = {k: v for k, v in attrs1.items() if "TPL" not in k}
        return attrs2

    @property
    def scopesim_dictionary(self):
        """
        Create ScopeSIM dictionary.
        """

        dd = {
            'class': self.__class__.__name__,
            'name': self.name,
            'target': self.target,
            'instrument': self.instrument,
            'description': self.__doc__.strip(),
            'templates': [
                template.scopesim_dictionary for template in self.templates
            ],
        }
        return dd
