# -*- coding: utf-8 -*-
"""
Recipes.
"""
from micado.main.DataItem import DataItem
from micado.main.Raw import Raw
from micado.main.MObject import MObject, persistent


class RecipeStep(MObject):
    """
    A RecipeStep.
    """

    dummy_recipestep: int = persistent("Dummy to fool database [None]", int, -42)

    @classmethod
    def get_previous_step(cls):
        """
        Get the previous step, if any.

        TODO: Just force the property name to be 'previous'?
        """
        prev_maybe = [
            getattr(cls, pp).prop_type
            for pp in cls.get_persistent_properties()
            if type(getattr(cls, pp).prop_type) != tuple and
            issubclass(getattr(cls, pp).prop_type, RecipeStep)
        ]
        assert len(prev_maybe) < 2
        prev = prev_maybe[0] if prev_maybe else None
        return prev

    @classmethod
    def get_raw(cls):
        """
        Get the raw data that is input to this class.

        TODO: Just force the property name to be 'raw'?
        """
        raw_maybe = [
            getattr(cls, pp).prop_type
            for pp in cls.get_persistent_properties()
            if type(getattr(cls, pp).prop_type) != tuple and
            issubclass(getattr(cls, pp).prop_type, Raw)
        ]
        assert len(raw_maybe) < 2
        raw = raw_maybe[0] if raw_maybe else None
        return raw

    @classmethod
    def get_other_inputdata(cls):
        """
        Get the input data that is not the previous recipe or a raw.

        TODO: What about recipes that start with output from previous recipes?
        TODO: What about class collisions?
        """
        cprev = cls.get_previous_step()
        craw = cls.get_raw()
        ins = [
            getattr(cls, pp).prop_type
            for pp in cls.get_persistent_properties()
            if type(getattr(cls, pp).prop_type) != tuple and
            issubclass(getattr(cls, pp).prop_type, DataItem) and
            getattr(cls, pp).prop_type not in [craw, cprev]
        ]
        return ins

    @classmethod
    def get_parameter_names(cls):
        """
        Get the input parameter names.
        """
        parnames = [
            pp
            for pp in cls.get_persistent_properties()
            # TODO: Simplify this
            if type(getattr(cls, pp).prop_type) == tuple or (
                not issubclass(getattr(cls, pp).prop_type, (DataItem, RecipeStep)) and
                pp not in ['object_id', 'name', 'filename', 'globalname'] and
                not pp.startswith('dummy')
            )
        ]
        return parnames
