# -*- coding: utf-8 -*-
"""
Templates.
"""

from collections import OrderedDict
from micado.main.MObject import MObject, persistent


class Template(MObject):
    """
    A Template.
    """

    dummy1: int = persistent("Dummy to fool database [None]", int, -4)

    def real_fits_keywords(self, ppp=None):
        """
        Overloaded to add CLS.
        """
        # TODO: or should this be a regular property?
        # TODO: Align with use of ID in ESO templates, conficts with Detector
        kws = super().real_fits_keywords(ppp=ppp)
        k = " ".join(ppp + ["CLS"])
        kws[k.upper()] = self.__class__.__name__
        return kws

    @property
    def scopesim_dictionary(self):
        """
        Create ScopeSIM dictionary.

        TODO: Refactor all these scopesim_dictionaries into something
              suitable for a base class.
        """
        attrnames = [
            attrname for attrname in self.get_persistent()
            if not attrname.startswith('dummy') and attrname not in [
                'object_id', 'name', 'filename', 'globalname',
            ]]

        attrs = OrderedDict([(
            attrname,
            self.__getattribute__(attrname)
        )
            for attrname in attrnames
        ])

        dd = {
            'class': self.__class__.__name__,
            'name': self.__class__.__name__,
            'description': self.__doc__.strip(),
            'kwargs': attrs,
        }
        return dd
