# -*- coding: utf-8 -*-
"""
Create full dictionary of ESO keywords.
"""

import glob
import json
import pathlib
import re

from astropy.io import fits


# TODO: Perhaps make a proper (data)class of these keywords?
def get_keyword_dict():
    """
    Get dictionary of keywords from json files.
    """
    dir_json = pathlib.Path(__file__).resolve().parent.parent.parent / "datainterface" / "yaml"
    globs = str(dir_json / "*.flat.json")
    fns = glob.glob(globs)
    kds = [kd for fn in fns for kd in json.load(open(fn))]
    kkeys = [kd['name'] for kd in kds]
    keys_extra = [k for k in kkeys if kkeys.count(k) > 1]
    assert keys_extra == [], keys_extra
    kds2 = {
        kd['name']: kd for kd in kds
    }
    return kds2


def verify_fitsheaders(filename: str):
    """Verify the headers in a FITS file."""
    with fits.open(filename) as ffits:
        print(ffits[0].header.tostring(sep="\n"))
        print(ffits[1].header.tostring(sep="\n"))

        for hk in ffits[0].header:
            hk2 = hk[4:] if hk.startswith("ESO ") else hk
            hk3 = re.sub("[\\d]", "i", hk2)
            assert hk2 in esokeywords or hk3 in esokeywords, f"Warning! {hk2} not in esokeywords"


esokeywords = get_keyword_dict()
